# Pantera - Web Pen-Test Proxy
#
# FILENAME      : panteraPPA.py
# CODER         : Simon Roses Femerling
# DATE          : 11/21/2006
# LAST UPDATE   : 11/21/2006
# ABSTRACT      : Pantera Passive Analyzer class for plugins
#       
# - Roses Labs Innovations (RL+I)
# Roses Labs
# http://www.roseslabs.com
#
# Copyright (c) 2003-2006 Roses Labs.
#
# You may not distribute, transmit, repost this software for commercial 
# purposes without Roses Labs written permission. 
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, publish,
# distribute the Software, and to permit persons to whom the Software 
# is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

'''
@author:       Simon Roses Femerling
@license:      GNU General Public License 2.0 or later
@contact:      pantera.proxy@gmail.com
@organization: OWASP / Roses Labs
'''

import re
import sys
import os
import imp
import sets

#############################################################################################
# PPA Plugins Classes
#############################################################################################

#############################################################################################
# FUNC     : class PPM
# PARAMS   : ...
# RETURN   : ...
# ABSTRACT : Passive plugins must use this class
class PPM:
    '''
    Pantera Passive Manager Class.
    '''
    
    #############################################################################################
    # FUNC     : def __init__
    # PARAMS   : ...
    # RETURN   : ...
    # ABSTRACT : Init 
    def __init__(self):
        '''
        Init class.
        '''
        self.result = 0
        self.result_data = sets.Set([])
        self.check_result_data = 0
        self.idata = sets.Set([]) # uniqueness
        self.type = ""
        self.set_have = ""
        self.obj = ""
        self.set_other_type = ""
        self.set_level = "info"
    # EOF: def __init__
        
    #############################################################################################
    # FUNC     : def GetResult
    # PARAMS   : ...
    # RETURN   : string
    # ABSTRACT : Return plugin result
    def GetResult(self):
        '''
        Return plugin result.
        
        @return: int
        '''
        return self.result
    # EOF: def GetResult
    
    #############################################################################################
    # FUNC     : def GetResultData
    # PARAMS   : ...
    # RETURN   : string
    # ABSTRACT : Return plugin result data
    def GetResultData(self):
        '''
        Return plugin result data.
        
        @return: list
        '''
        return self.result_data
    # EOF: def GetResultData
    
    #############################################################################################
    # FUNC     : def SetResult 
    # PARAMS   : value
    # RETURN   : ...
    # ABSTRACT : Set plugin result
    def SetResult(self, r, h):
        '''
        Set plugin result.
        
        @type r: int
        @param r: Set plugin result.
        @type h: string
        @param h: Set have.
        '''
        if not (r == 1 or r == 0):
            self.result = 0
        else:
            self.result = r
        self.set_have = h
    # EOF: def SetResult
    
    #############################################################################################
    # FUNC     : def SetResultData
    # PARAMS   : string
    # RETURN   : ...
    # ABSTRACT : Set plugin result data
    def SetResultData(self, d):
        '''
        Set plugin result data.
        
        @type d: string
        @param d: Data.
        '''
        
        t = "%s(<>)%s"  % (self.set_level,d)
        self.result_data.add(t)
        self.check_result_data = 1
    # EOF: def SetResultData 
    
    #############################################################################################
    # FUNC     : def GetCheckResultData
    # PARAMS   : ...
    # RETURN   : string
    # ABSTRACT : Return result data
    def GetCheckResultData(self):
        '''
        Return result data.
        
        @return: string
        '''
        return self.check_result_data
    # EOF: def GetCheckResultData
    
    #############################################################################################
    # FUNC     : def SetType
    # PARAMS   : string
    # RETURN   : string...
    # ABSTRACT : Set type
    def SetType(self, t):
        '''
        Set type.
        
        @type t: string
        @param t: Type.
        '''
        self.type = t
    # EOF: def SetType
    
    #############################################################################################
    # FUNC     : def SetTypeData
    # PARAMS   : string
    # RETURN   : ...
    # ABSTRACT : Set data type
    def SetTypeData(self, v):
        '''
        Set data type.
        
        @type v: string
        @param v: Data type.
        '''
        self.idata.add(v)
    # EOF: def SetTypeData
        
    #############################################################################################
    # FUNC     : def GetType
    # PARAMS   : ...
    # RETURN   : string
    # ABSTRACT : Return type
    def GetType(self):
        '''
        Return type.
        
        @return: string
        '''
        return self.type
    # EOF: def GetType
    
    #############################################################################################
    # FUNC     : def GetTypeData
    # PARAMS   : ...
    # RETURN   : list
    # ABSTRACT : Return type data
    def GetTypeData(self):
        '''
        Return type data.
        
        @return: list
        '''
        return self.idata
    # EOF: def GetTypeData
    
    #############################################################################################
    # FUNC     : def GetHave
    # PARAMS   : ...
    # RETURN   : string
    # ABSTRACT : Return have.
    def GetHave(self):
        '''
        Return have.
        
        @return: string
        '''
        return self.set_have
    # EOF: def GetHave
    
    #############################################################################################
    # FUNC     : def SetObjData
    # PARAMS   : obj
    # RETURN   : ...
    # ABSTRACT : Set object
    def SetObjData(self, o):
        '''
        Set object.
        
        @type o: obj
        @param o: Set object.
        '''
        self.obj = o
    # EOF: def SetObjData

    #############################################################################################
    # FUNC     : def SetOtherType
    # PARAMS   : string
    # RETURN   : ...
    # ABSTRACT : Add dditional have
    def SetOtherType(self, h):
        '''
        Add additional have.
        
        @type h: string
        @param h: Additional have.
        '''
        self.set_other_type = h
    # EOF: def SetOtherType
        
    #############################################################################################
    # FUNC     : def GetOtherType
    # PARAMS   : ...
    # RETURN   : string
    # ABSTRACT : Return other type.
    def GetOtherType(self):
        '''
        Return other type.
        
        @return: string
        '''
        return self.set_other_type
    # EOF: def GetOtherType

    #############################################################################################
    # FUNC     : def SetLevel
    # PARAMS   : string
    # RETURN   : ...
    # ABSTRACT : Set level 
    def SetLevel(self, l="info"):
        '''
        Set level.
        
        @type l: string
        @param l: Set level.
        '''
        if l == "":
            self.set_level = "info"
        elif l == "info":
            self.set_level = "info"
        elif l == "low":
            self.set_level = "low"
        elif l == "med":
            self.set_level = "med"
        elif l == "high":
            self.set_level = "high"
        elif l == "safe":
            self.set_level = "safe"
        else:
            self.set_level = "info"
    # EOF: def SetLevel

    ###################################################
    # Plugins Public Functions
    ###################################################
        
    #############################################################################################
    # FUNC     : def InitAnalyzer
    # PARAMS   : ...
    # RETURN   : ...
    # ABSTRACT : Init func inside plugin
    def InitAnalyzer(self):
        '''
        Init function inside plugin.
        '''
        pass
    # EOF: def InitAnalyzer
    
    #############################################################################################
    # FUNC     : def BeginAnalyzer
    # PARAMS   : ...
    # RETURN   : ...
    # ABSTRACT : Begin func inside plugin
    def BeginAnalyzer(self,obj):
        '''
        Begin function inside plugin.
        
        @type obj: obj
        @param obj: Set object.
        '''
        pass
    # EOF: def BeginAnalyzer
    
# RL+I EOF
