# Pantera - Web Pen-Test Proxy
#
# FILENAME      : panteraStatus.py
# CODER         : Simon Roses Femerling
# DATE          : 11/23/2006
# LAST UPDATE   : 11/23/2006
# ABSTRACT      : Pantera Status Class
#       
# - Roses Labs Innovations (RL+I)
# Roses Labs
# http://www.roseslabs.com
#
# Copyright (c) 2003-2006 Roses Labs.
#
# You may not distribute, transmit, repost this software for commercial 
# purposes without Roses Labs written permission. 
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, publish,
# distribute the Software, and to permit persons to whom the Software 
# is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

'''
@author:       Simon Roses Femerling
@license:      GNU General Public License 2.0 or later
@contact:      pantera.proxy@gmail.com
@organization: OWASP / Roses Labs
'''

import re
import sys
import os
import imp
import sets

#############################################################################################
# Classes
#############################################################################################

#############################################################################################
# FUNC     : class PanteraStatus
# PARAMS   : ...
# RETURN   : ...
# ABSTRACT : keep track of plugins status
class PanteraStatus:
    '''
    Pantera Status Manager Class.
    '''
    
    #############################################################################################
    # FUNC     : def __init__
    # PARAMS   : ...
    # RETURN   : ...
    # ABSTRACT : Init 
    def __init__(self):
        '''
        Init class.
        '''
        self.iClear()
    # EOF: def __init__
        
    def iClear(self):
        # status variables
        self.error_msg = ''
        self.status_list = []
        self.status_dict = {}
        self.max_len = 200
        self.uni_pid = sets.Set([])
        """
        status_dict struct
        
        status_dict['pid'] = ImportExport, Report, Garra, etc.
        status_dict['id'] = plugin id
        status_dict['desc'] = plugin desc
        status_dict['status'] = finish, working , error, waiting
        status_dict['options'] = plugin options save
        """
        
   #############################################################################################
    # FUNC     : 
    # PARAMS   : 
    # RETURN   : 
    # ABSTRACT : 
    def GetErroMsg(self):
        '''
    
        '''
        return self.error_msg

    #############################################################################################
    # FUNC     : 
    # PARAMS   : 
    # RETURN   : 
    # ABSTRACT : 
    def SetErrorMsg(self,m):
        '''
    
        '''
        self.error_msg = m
        
    def GetMaxLen(self):
        return self.max_len
    
    def SetMaxLen(self, m):
        self.max_len = m
        
    def StatusLen(self):
        return len(self.status_list)
    
    def GetStatusPos(self, p):
        l = self.StatusLen()
        if p > l or p < 0:
            return {}
        return self.status_list[p]
    
    def GetStatus(self, p):
        d = self.GetStatusPos(p)
        if d == {}: return "","","","",""
        return d['pid'],d['id'],d['status'],d['desc'],d['options']
        
    def GetStatusList(self):
        return self.status_list
 
    def GetStatusListDict(self, pid, id):
        d = {}
        for i in self.GetStatusList():
            if i['pid'] == pid and i['id'] == id:
                d = i
                break
        return d
       
    def GetPos(self, pid, id):
        c = -1
        for i in self.GetStatusList():
            if i['pid'] == pid and i['id'] == id:
                c += 1
                break
        return c
    
    def InsertStatusDict(self,d):
        c = self.Check(d['pid'],d['id'])
        self.uni_pid.add(d['pid'])
        if self.StatusLen == self.max_len:
            self.DelStatusPos(0)
        if c == 0:
            self.status_list.append(d)
        else:
            p = self.GetPos(d['pid'],d['id'])
            self.DelStatusPos(p)
            self.status_list.append(d)
            
    def InsertStatus(self, pid='', id='', status='', desc='',options=''):
        d = {}
        d['pid'] = pid
        d['id'] = id
        d['status'] = status
        d['desc'] = desc
        d['options'] = options
        self.InsertStatusDict(d)    
    
    def InsertOptions(self,pid,id,options):
        return self.iUpdate(pid, id, 'options',options)
    
    def DelStatusPos(self, p):
        l = self.StatusLen()
        if p > l or p < 0:
            return -1
        del self.status_list[p]
        return 0
    
    def updateStatus(self,pid,id,status):
        return self.iUpdate(pid, id, 'status',status)
    
    def iUpdate(self, pid, id, key, val):
        c = 0
        f = 0
        self.uni_pid.add(pid)
        for i in self.GetStatusList():
            if i['pid'] == pid and i['id'] == id:
                i[key] = val
                self.DelStatusPos(c)
                self.InsertStatusDict(i)
                f = 1
                break    
            c += 1
        if f == 0:
            d = {}
            d['pid'] = pid
            d['id'] = id
            d[key] = val
            self.InsertStatusDict(d)
        return 
        
    def Check(self, pid, id):
        c = 0
        for i in self.GetStatusList():
            if i['pid'] == pid and i['id'] == id:
                c = 1    
                break
        return c
                    
    def ReturnPid(self):
        return self.uni_pid
    
    def GetOptions(self, pid, id):
        d = {}
        c = self.Check(pid,id)
        if c == 1:
            i = self.GetStatusListDict(pid,id)
            if i.has_key('options'):
                d = i['options']
        return d
                    
# RL+I EOF
