# Pantera Import Export Plugin - Paros Parser
#
# FILENAME      : paros_parser.py
# CODER         : Simon Roses Femerling
# DATE          : 11/08/2006
# LAST UPDATE   : 11/13/2006
# ABSTRACT      : Import Paros Proxy Data
#                 Ported/Modified version of Paros Parser from Oedipus Scanner
#       
# - Roses Labs Innovations (RL+I)
# Roses Labs
# http://www.roseslabs.com
#
# Copyright (c) 2003-2006 Roses Labs.
#
# You may not distribute, transmit, repost this software for commercial 
# purposes without Roses Labs written permission. 
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, publish,
# distribute the Software, and to permit persons to whom the Software 
# is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

'''
@author:       Simon Roses Femerling
@license:      GNU General Public License 2.0 or later
@contact:      pantera.proxy@gmail.com
@organization: OWASP / Roses Labs
'''

from panteraImportExport import PIE
import re
import string

plugin_info =  {'id':'ParosParser',
                'desc':'Paros Proxy Log.',
                'author':'Simon Roses Femerling',
                'email':'sroses@roseslabs.com',
                'version':'0.1',
                'options':'no',
                'type':'import'  # import or export type
                } 

class ParosParser(PIE):
    '''
    ParosParser class.
    '''
    
    def InitIE(self):
        '''
        Init function.
        '''
        PIE.__init__(self)
        self.url = ''
        self.ip = ''
        self.port = "80"
        self.ssl = 0
        self.TIME = re.compile('^(\d\d?:\d\d?:\d\d?)\s[A|P]M')
        self.GET = re.compile('^GET')
        self.POST = re.compile('^POST')
        self.HTTP = re.compile('^HTTP')
        self.HEADER = re.compile('\w(-\w)*:')
        self.DELIM = re.compile('^==== \d+ ==========')
        self.file_data = []
        self.request_buf = ''
        self.response_buf = ''
        self.all_data = []
        return 0
        
    def BeginIE(self):
        '''
        Begin function.
        '''
        er = self.OpenFile()
        if er == -1:
            return -1
        self.file_data = self.ReadFileIntoList()
        if self.file_data == []:
            return -1
        self.iParser()
        self.CloseFile()
        return 0
                
    def iParser(self):
        linenum = 0
        while linenum < len(self.file_data):
            line = self.file_data[linenum]
            if re.match(self.GET,line):
                linenum = self.DoRequest(linenum)
            elif re.match(self.POST,line):
                linenum = self.DoRequest(linenum)
            elif re.match(self.HTTP,line):
                linenum = self.DoResponse(linenum)
            linenum += 1
            if self.request_buf != '' and self.response_buf != '':
                self.InsertIntoFramework(self.request_buf,self.response_buf)
                self.request_buf = ''
                self.response_buf = ''
                
    def DoRequest(self, linenum):
        
        self.request_buf = ''
        
        self.request_buf += string.strip(self.file_data[linenum]) + '\r\n'
        linenum += 1
        
        while linenum < len(self.file_data):
            line = self.file_data[linenum]
            if re.search(self.HEADER,line):
                self.request_buf += string.strip(line) + '\r\n'
                linenum += 1
            else:
                break
        # done with parsing headers...
        
        linenum += 1

        lin = self.file_data[linenum]
        if lin != '':
            self.request_buf += '\r\n' + string.strip(self.file_data[linenum]) # Request body
        return linenum
                
    def DoResponse(self, linenum):

        self.response_buf = ''
        
        self.response_buf += string.strip(self.file_data[linenum]) + '\r\n'
        linenum += 1
        
        while linenum < len(self.file_data):
            line = self.file_data[linenum]
            if re.search(self.HEADER,line):
                self.response_buf += string.strip(line) + '\r\n'
                linenum += 1
            else:
                break
        # done with parsing headers...
        
        while linenum < len(self.file_data):
            line = self.file_data[linenum]
            if re.search(self.DELIM,line):
                break
            else:
                self.response_buf += string.strip(line) + '\r\n'
                linenum += 1
        # done with parsing body...

        return linenum
                        
# RL+I EOF